/*****************************************************************

docbegin:

*  Name of source file:         j_src_lc_bin.c

*  Version of source file:      0.01

*  Parent component:            None

*  Programmer:                  Stefan Larsson and Sara Janek

*  Affiliation:                 Stockholm Observatory

*  Contact address:             Stockholm Observatory
*   			        SE-133 36 Saltsjobaden, Sweden
*                               Phone: +46 8-164464  FAX: +46 8-7174719
*                               stefan@astro.su.se

*  Purpose:                     Bin together event weights to produce
*				background subtracted source light curves. 
*				Correct for correlations between sources
*				and between sources and background.

*  Origin date:                 000606

*  Update history:              
   
*  Type of element:             Function

*  Part of package:             j_src_lc

*  Name:                        j_src_lc_bin

*  Calls:                       

*  Returns:			status

*  Required components:         DAL, PIL, RIL, DAL3GEN


docend:

Various remarks:


Flow of program:

Error codes:

Basic offset:

*****************************************************************/

#include <j_src_lc.h>

int j_src_lc_bin( dal_element *ptr_inSWG,         /* Pointer to input SWG */
	      JEMX_num jemxNum,                    /* Number of JEMX unit */
	      int numSources,     	           /* Number of sources  */
	      int nChanBins,                     /* Number of energy bins */
	      int chanLow[J_SRCEXTR_MAXNCHANLC], /* Lower energy limits */
	      int chanHigh[J_SRCEXTR_MAXNCHANLC],/* Upper energy limits */
	      double timeStep,		      /* Length of time bin */
	      OBTime OBTstart,                /* Start time for lightcurve */
	      OBTime OBTend,                  /* End time for lightcurve */
	      float *eventWeights	      /* Event weights */
	      int numEvents,		      /* Number of events */
	      int status )                    /* Status flag */
{
  /* Ordinary variables */
  
  void *buff_timeOBT = NULL;
  void *buff_qualityFlag = NULL;
  void *buff_rate = NULL;
  void *buff_staterr = NULL;
  void *buff_syserr = NULL;
  
  const char tableName[15];
  const char colName[15];
  
  double rateIllum[numSources][nChanBins];
  double rateShad[numSources][nChanBins];
  double rate[numSources][nChanBins];
  double rateErrorSys[numSources][nChanBins];    
  double rateErrorStat[numSources][nChanBins];
  
  int i;
  int iEvent;      /* Index over events  */
  int isource;     /* Index over sources  */
  int eband;       /* Index over energy bands  */
  int qualityFlag[nChanBins];
  
  long buffSizeOBTime;
  long buffSizeInt;
  long buffSizeDouble;
  long iTimeBin;
  
  OBTime *timeOBT;
  OBTime OBTStartBin; 
  OBTime OBTEndBin;
  
  dal_element *ptrTbl = NULL;
  
  dal_dataType DALtype;
  
  dal_accessType aType;
  
  /* End of DAL etc. declarations */
  
  /*  Immediate return if called while executable is in error condition  */
  
  if( status != ISDC_OK ) return( status ); 
  
 
  /* Set up data structure names */
  
  if( jemxNum == 1 ){
    strcpy( tableName,"JMX1-SRLC-LCV" );
  }
  else if ( jemxNum == 2 ){
    strcpy( tableName,"JMX2-SRCL_LCV" );
  }
  else {
    status = RILlogMessage( NULL,Error_1,
			    "JEM-X instrument number %i impossible!",jemxNum,status );
  }
  
  
  /* Create a new table element */
  
  
  aType = DAL_DISC;
  status = DALtableCreate( ptr_inSWG,aType,( long )numEvents, 
			   tableName,&ptrTbl,status ); 
  if( status != ISDC_OK ) {
    status = RILlogMessage( NULL,Log_0, 
			    "Cannot find requested Tbl: %s",tableName ); 
  }
  else return( status );
  
 
  /*
   *
   * Add columns RATE, STAT_ERR and SYS_ERR to the new table 
   *
   */

  DALtype = DAL_DOUBLE;  
  for( i = 0; i < numSources; i++ ){
    
    strcpy( colName,"RATE_" );
    strcat( colName,( const char )i );
    
    status = DALtableAddCol( ptrTbl,colName,DALType,( long )nChanBins,status );
    if( status!= ISDC_OK ){
      status = RILlogMessage( NULL,Log_0,"Cannot add new column: %s to table %s.",
			      colName,&ptrTbl,status );
    }
    else return( status );
    
    strcpy( colName,"STAT_ERR_" );
    strcat( colName,( const char )i );
    
    status = DALtableAddCol( ptrTbl,colName,DALType,( long )nChanBins,status );
    if( status!= ISDC_OK ){
      status = RILlogMessage( NULL,Log_0,"Cannot add new column: %s to table %s.",
			      colName,&ptrTbl,status );
    }
    else  return( status );		    
    
    strcpy( colName,"SYS_ERR_" );
    strcat( colName,( const char )i );
    
    status = DALtableAddCol( ptrTbl,colName,DALType,( long )nChanBins,status );
    if( status!= ISDC_OK ){ 
      status = RILlogMessage( NULL,Log_0,"Cannot add new column: %s to table %s.",
			      colName,&ptrTbl,status );
    else return( status );	  
    } 
  }
  
    
    /*
     *
     * Add column QUALITY to the table 
     *
     */

    DALtype = DAL_INT;
    strcpy( colName,"QUALITY" );
    
    status = DALtableAddCol( ptrTbl,colName,DALType,( long )nChanBins,status );
    if( status!= ISDC_OK ){
      status = RILlogMessage( NULL,Log_0,"Cannot add new column: %s to table %s.",
			      colName,&ptrTbl,status );
    else return( status );	  
    }
    
  
    /*
     *
     * Allocate memory for the transfer buffers QUALITY, RATE, STAT_ERR and SYS_ERR 
     *
     */

    status = DALallocateDataBuffer( &buff_qualityFlag,nChanBins*sizeof( int ),status );
    status = DALallocateDataBuffer( &buff_rate,nChanBins*sizeof( double ),status );
    status = DALallocateDataBuffer( &buff_staterr, nChanBins*sizeof( double ), status );
    status = DALallocateDataBuffer( &buff_syserr,nChanBins*sizeof( double ), status );
  
    /* Restructure arrays */
    
    timeOBT = buff_timeOBT;
    qualityflag = buff_qualityFlag;
    
    
    /*  Set time limits for first bin  */
    
    OBTStartBin = OBTStart;
    OBTEndBin = OBTStart + timeStep;
    
    
    
    /*  Main loop over all events  */  
    
    for( iEvent = 0L; iEvent < numEvents; iEvent++ ) {  
      
      if(eventOBT[iEvent] < OBTStartBin) {
	iEvent = iEvent + 1;
      }
      elseif(eventOBT[iEvent] > OBTStartBin && eventOBT[iEvent] < OBTEndBin) { 
	
	/*  Loop over sources and energy bands  */
	
	for( isource=0L; isource < numSources; isource++ ) {  
	  
	  /*  Add event contribution to source/energy bin  */
	  /* Note: If energy bands are overlaping an event can
	     contribute to more than one band.  */
	  
	  for( eband=0L; eband < nChanBins; eband++ ) {  
	    if(pi[iEvent] >= chanLow[eband] && pi[iEvent] <= chanHigh[eband]){
	      rate[isource][eband] = rate[isource][eband] + eventWeight[isource];
	    }
	  }
	}  /*  End of loop over sources and energy bands  */
      }
      elseif(eventOBT[iEvent] > OBTEndBin) {
	
	/* If end of time bin write to datastructure and reset
	   rate--variables. Increment timebin parameters.  */
	
	
	timeOBT = OBTStartBin + 0.5*timeStep;  /* Center of time bin */
	
	/*  Loop over sources and energy bands to calculate
	    error estimates.  */
	
	for( isource=0L; isource < numSources; isource++ ) {  
	  
	  for( eband=0L; eband < nChanBins; eband++ ) { 
	    rateErrorStat[isource][eband] =  1.0;   /* Dummy value */
	  }
	}  /*  End of loop over sources and energy bands  */
	
	/* Remember to increment OBTStartBin & OBTEndBin */
	/* Set iTimeBin somewhere */
	
	
	
	/* 
	 *
	 * Write single light curve to output data structure 
	 *
	 */
  
	status = DAL3GENtablePutOBTBins( ptrTbl,"OB_TIME",0,iTimeBin,iTimeBin, 
					 ( long )nChanBins,timeOBT,status );
	if( status != ISDC_OK ){
	  status = RILlogMessage( NULL,Error_3,
				  "Problem writing OB_TIME column!",status);
	  break;
	}
	else return( status );
	
	DALtype = DAL_INT;
	status = DALtablePutColBins( ptrTbl,"QUALITY",0,DALType,iTimeBin,iTimeBin, 
				     ( long )nChanBins,buff_qualityFlag,status );
	if( status!= ISDC_OK ){ 
	  status = RILlogMessage( NULL,Error_3,
				  "Problem writing QUALITY column!",status);
	  break;
	}
	else return( status );
	


	DALtype = DAL_DOUBLE;
	for(i = 0L; i< numSources; i++){
	  strcpy( colName,"RATE_" );
	  strcat( colName,( const char )i );
	  rate[i] = buff_rate;
	  status = DALtablePutColBins( ptrTbl,colName,0,DALType,iTimeBin,iTimeBin,
				   ( long )nChanBins,buff_rate,status );
	  if( status!= ISDC_OK ){
	    status = RILlogMessage( NULL,Error_3,
				    "Problem writing RATE_%s column!",i,status);
	    break;
	  }
	  else return( status );
	  
      
	  strcpy( colName,"STAT_ERR_" );
	  strcat( colName,( const char )i );
	  rateErrorStat[i] = buff_staterr;
	  status = DALtablePutColBins( ptrTbl,colName,0,DALType,iTimeBin,iTimeBin,
				       ( long )nChanBins,buff_staterr,status );
	  if( status!= ISDC_OK ){
	    status = RILlogMessage( NULL,Error_3,
				    "Problem writing STAT_ERR_%s column!",i,status);
	    break;
	  }
	  else return( status );
	     
      
	  strcpy( colName,"SYS_ERR_" );
	  strcat( colName,( const char )i );
	  if( rateErrorSys[i] == NULL ){
	    status = DALtableSetNullValues( ptrTbl,colName,0, 
					    iTimeBin,iTimeBin,status );
	  }
	  else{
	    rateErrorSys[i] = buff_syserr;
	    status = DALtablePutColBins( ptrTbl,colName,0,DALType,iTimeBin,iTimeBin,
					 ( long )nChanBins,buff_syserr,status );
	  }
	  if( status!= ISDC_OK ){ 
	    status = RILlogMessage( NULL,Error_3,
				    "Problem writing SYS_ERR_%s column!",i,status);
	    break;
	  }
	  else return( status );
	    
	}
	
	

	/* De-allocate buffer memory */

	if( buff_qualityflag != NULL ) { status = DALfreeDataBuffer( buff_qualityflag,status ); }
	if( buff_rate != NULL ) { status = DALfreeDataBuffer( buff_rate,status ); }
	if( buff_staterr != NULL ) { status = DALfreeDataBuffer( buff_staterr,status ); }
	if( buff_syserr != NULL ) { status = DALfreeDataBuffer( buff_syserr,status ); }
	
	if( status != ISDC_OK ) return( status );
	if( chatty > J_CHATTY_NORMAL ){
	  status = RILlogMessage( NULL,Log_0,
				  "work ##26## Buffer memory has been freed",status);
	}
  
	
    
   
	/*
	 *
	 * Write keywords CHSTRTnn, CHSTOPnn in the output header
	 *
	 */
	
	
	for( i=0L; i < numSources; i++ ){
	  strcpy( colName,"CHSTRTnn_" );
	  strcat( colName,( const char )i );
	  status = DALattributePutInt( ptrTbl,colName,( long )chanLow[i],
				   " ","Lower energy limit",status );
	  if( status != ISDC_OK ){
	    status = RILlogMessage( NULL,Log_0, 
				    "Problem writing keywords in the output header ",status );
	  }
	  else return( status );
      
      
	  strcpy( colName,"CHSTOPnn_" );
	  strcat( colName,( const char )i );
	  status = DALattributePutInt( ptrTbl,"CHSTOPnn",( long )chanHigh[i], 
				   " ","Upper energy limit",status );
	  if( status != ISDC_OK ){
	    status = RILlogMessage( NULL,Log_0, 
				    "Problem writing keywords in the output header ",status );
	  }
	  else return( status ); 
	}
    
  
	/* 
	 *
	 * Write keywords NSOURCES, NUMBAND and TIMDEL in the output header 
	 *
	 */

	status = DALattributePutInt( ptrTbl,"NSOURCES",( long )numSources, 
				     " ","Number of sources",status );
	status = DALattributePutInt( ptrTbl,"NUMBAND",( long )nChanBins, 
				 " ","Number of energy bands",status);
	if( status != ISDC_OK ){
	  status = RILlogMessage( NULL,Log_0, 
				  "Problem writing keywords in the output header ",status );
	}
	else return( status );
  
	status = DALattributePutReal( ptrTbl, "TIMEDEL", timeStep,  
				      "s", "Integration time", status ); 
	if( status != ISDC_OK ){
	  status = RILlogMessage( NULL,Log_0, 
				  "Problem writing keywords in the output header ",status );
	}
	else return( status );
	
		
	/* 
	 *
	 * Write keywords INSTRUME, BASETYPE, TELESCOP in the output header 
	 *
	 */
	
	if( jemxNum == 1 ){
	  strcpy( tableName, "JEM-X_1" );
	}
	else if( jemxNum == 2 ){
	  strcpy( tableName, "JEM-X_2" );
	}
	else{
	  status = RILlogMessage( NULL,Error_1,
				  "JEM-X instrument number %i impossible!",jemxNum,status );
	}
	
	status = DALattributePutChar( ptrTbl,"INSTRUME",tableName," ",
				      "Instrument name",status ); 
	status = DALattributePutChar( ptrTbl,"BASETYPE","DAL_TABLE"," ", 
				      "DAL base typr of the data structure",status );
	status = DALattributePutChar( ptrTbl,"TELESCOP","INTEGRAL"," ",
				      "Instrument name",status );
	if( status != ISDC_OK ){ 
	  status = RILlogMessage( NULL,Log_0,  
				  "Problem writing keywords in the output header",status );
	}
	else return( status );			
      } 
      
      return( status );
      
    }
}










