/*****************************************************************

docbegin:


*  Name of source file:         j_src_lc_params.c

*  Version of source file:      0.01

*  Parent component:            None

*  Programmer:                  Stefan Larsson

*  Affiliation:                 Stockholm Observatory

*  Contact address:             Stockholm Observatory
*   			        SE-133 36 Saltsjobaden, Sweden
*                               Phone: +46 8-164464  FAX: +46 8-7174719
*                               stefan@astro.su.se

*  Purpose:                     Get the parameters and open files for the
*                               j_src_extr executable

*  Origin date:                 000511

*  Update history:              0.01    Adapted from 
                                        j_ima_basic_recon_params.c and
                                        jextr5GetParams.c
*                                                 
   
*  Type of element:             Function

*  Part of package:             j_src_extr

*  Name:                        j_src_extr_params

*  Calls:

*  Returns:

*  Required components:         DAL, PIL, RIL, DAL3GEN

docend:

Various remarks:

Flow of program:

*****************************************************************/

#include <j_src_extr.h>

int j_src_lc_params(
         dal_element **ptr_inSWG, /*  Pointer to SWG                */
         JEMX_num *jemxNum,       /*  JEM-X unit number             */
         JEMX_type *jemxType,     /*  Type of events (FULL or REST) */
         double radDetector,	  /*  Detector radius		    */
         double radMask,	  /*  Mask radius		    */
         double detMaskDistance,  /*  Detector mask distance	    */
         double rhoCell,          /*  Mask cell size		    */
         int *chatty,             /*  Level of debugging output     */
         int status )             /*  Status flag                   */
{

   /*  Ordinary variables  */

   void *arrayPointer = NULL;
   void *arrayBuff = NULL;

   char inSWG[DAL_MAX_NAME_SIZE];
   char JMX1instData[DAL_MAX_NAME_SIZE];
   char JMX2instData[DAL_MAX_NAME_SIZE];
   char JEMXdata[DAL_MAX_NAME_SIZE];
   char structName[DAL_MAX_NAME_SIZE];
   char strjemxNum[5];
   char templateName[DAL_MAX_NAME_SIZE];

   int numAxes;
   int numChildren;

   long loop;
   long axes[DAL_MAX_ARRAY_DIMENSION];
   long startValues[2];
   long endValues[2];
   long numValues;
   long numberOfCounts;
   long n_closed = 0;
   long n_open = 0;

   /*  DAL, RIL, PIL, COMMON declarations  */

   dal_element  *shgramPtr = NULL;    /*  Pointer to shadogram DS  */

   dal_dataType DALtype;

   dal_accessType  accessType;

   dal_baseType    baseType;

   sw_type    type_sw;

   Instrument   inst;

   /*  End of DAL etc. declarations  */

   /*  Immediate return if called while executable is in error condition  */

   if( status != ISDC_OK ) return( status );


      /*  Get the parameters  */

   status = PILGetString("inSWG", inSWG);
   if( status != ISDC_OK ) return( status );
   status = RILlogMessage( NULL, Log_0,
                "Value of inSWG: %s", inSWG);
   if( status != ISDC_OK ) return( status );

   status = PILGetString("JMX1instData", JMX1instData);
   if( status != ISDC_OK ) return( status );
   status = RILlogMessage( NULL, Log_0,
                "Value of JMX1instData: %s", JMX1instData);
   if( status != ISDC_OK ) return( status );

   status = PILGetString("JMX2instData", JMX2instData);
   if( status != ISDC_OK ) return( status );
   status = RILlogMessage( NULL, Log_0,
                "Value of JMX2instData: %s", JMX2instData);
   if( status != ISDC_OK ) return( status );

   status = PILGetInt("jemxType", jemxType);
   if( status != ISDC_OK ) return( status );
   if( *chatty > J_CHATTY_QUIET ) {
      status = RILlogMessage( NULL, Log_0,
                "parm ##3## Value of jemxType: %d", *jemxType);
   }

   status = PILGetInt("chatty", chatty);
   if( status != ISDC_OK ) return( status );
   status = RILlogMessage( NULL, Log_0,
                "Value of chatty: %d", *chatty);
   if( status != ISDC_OK ) return( status );

   status = PILGetInt("jemxNum", jemxNum);
   if( status != ISDC_OK ) return( status );
   status = RILlogMessage( NULL, Log_0,
                "Value of jemxNum: %d", *jemxNum);
   if( status != ISDC_OK ) return( status );

   /*  Check the value of jemxNum: should be 1 or 2  */

   if( *jemxNum != 1 && *jemxNum != 2 ) {
      status = J_ERROR_CODE_START + J_SRCEEXTRACT_ERROR_START
               + J_EXTR_ERROR_START + J_EXTR_WRONG_JEMX_NUM;
      return( status );
   }
   
   /*  Check the value of jemxType: should be FULL(0) or REST(1)  */

   if( *jemxType != FULL && *jemxType != REST ) {
      status = J_ERROR_CODE_START + J_SAIMAGE_ERROR_START
               + J_ISG_ERROR_START + J_ISG_WRONG_TYPE;
      return( status );
   }

/*
 *   Compose the strings: strjemxNum and JEMXdata
 */

   if( *jemxNum == 1 ) {
       strcpy( strjemxNum, "JMX1" );
       strcpy( JEMXdata, JMX1instData );
   }
   if( *jemxNum == 2 ) {
       strcpy( strjemxNum, "JMX2" );
       strcpy( JEMXdata, JMX2instData );
   }

/*
 *
 *     Get the mask pattern
 *
 */

   /*  Create structure name of JEMX instr. data : mask layout  */

   strcpy( structName, strjemxNum );
   strcat( structName, "-MASK-MOD" );

   status = j_ibr_get_jemx_arr_short(structName, JEMXdata,
                           maskPatternArray, status);
   if( status != ISDC_OK ) return( status ); /*  On error: return  */

   /*  verify contents of mask pattern  */

   for( loop = 0L; loop < 40000L; loop++) {
       if( maskPatternArray[loop] == J_EXTR_CLOSED_CELL ) n_closed++;
       if( maskPatternArray[loop] == J_EXTR_OPEN_CELL   ) n_open++;
   }
   if( *chatty > 1 )
   printf("parm ##0## Number open/closed cells : %ld/%ld\n", n_open, n_closed);

/*
 *
 *     Get other instrument parameters
 *
 */


 */

/*
 *
 *       Open the SWG
 *
 */



 

   if( *chatty > J_CHATTY_NORMAL ) {
         status = RILlogMessage( NULL, Log_0,
      "parm ##8## The SWG to open: %s", inSWG);
   }
   status = DALobjectOpen(inSWG, ptr_inSWG, status);
   if( status != ISDC_OK ) return( status ); /*  On error: jump to end  */
   if( *chatty > J_CHATTY_NORMAL ) {
      status = RILlogMessage( NULL, Log_0,
      "parm ##9## DAL entrance group has now been opened");
   }




   return( status );

}

